/*
 * Copyright 1996 by Michel D. Grimminck
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This file modified for simplicity and improved execution speed
 * by Harm Jetten, 1998.
 */

#include "var.h"

INDEX combi_array[51][8];

void init_index()
{
  int a,b;

  combi_array[0][0]=1;
  for(a=1;a<=50;a++) {
    combi_array[a][0]=1;
    for(b=1;b<8;b++)
      combi_array[a][b]=combi_array[a-1][b-1]+combi_array[a-1][b];
  }
}

INDEX db_count(int wman,int wcrown,int bman,int bcrown)
{
  return combi_array[50-bman-wman-bcrown][wcrown]*
         combi_array[50-bman-wman][bcrown]*
         combi_array[45][wman]*
         combi_array[45][bman];
}

INDEX index_singletype(int n,int offset,int fields,int *pos)
/* pos must be a strictly increasing function, with a maximum of 'fields-1' */
{
  INDEX result=0;
  int i,lp;

  for(i=0;i<n;i++) {
    lp=pos[i]-offset;
    result+=combi_array[fields][n-i]-combi_array[fields-lp][n-i];
    fields-=lp+1; offset=pos[i]+1;
  }
  return result;
}
    
#define wc (white|crown)
#define bc (black|crown)
#define wm (white|man)
#define bm (black|man)

INDEX database_linear_index(int color)
/* reads the 'board' array and determines the index function (not in bytes)
   method a la Wiesenekker
*/
{
  INDEX index,p1,p2,p3;
  int p,i,j,jumpover;
  int np[8]={0,0,0,0,0,0,0,0},list[8][8];
  int decrease[8][8];

  /* make sorted lists per piece type */
  if (color==white) for(p=0;p<50;p++) switch(board[map[p]]) {
    case wm: list[wm][np[wm]++]=p;break;
    case wc: list[wc][np[wc]++]=p;break;
    case bm: list[bm][np[bm]++]=p;break;
    case bc: list[bc][np[bc]++]=p;break;
  } else for(p=0;p<50;p++) switch(board[map[49-p]]) {
    case bm: list[wm][np[wm]++]=p;break;
    case bc: list[wc][np[wc]++]=p;break;
    case wm: list[bm][np[bm]++]=p;break;
    case wc: list[bc][np[bc]++]=p;break;
  }

  /* set jumpovers white man */
  for(i=0;i<np[wm];i++) {
    jumpover=0;
    for(j=0;j<np[bm];j++) if (list[bm][j]<list[wm][i] && list[bm][j]>=5) jumpover++;
    decrease[wm][i]=jumpover;
  }
  /* set jumpovers black crown */
  for(i=0;i<np[bc];i++) {
    jumpover=0;
    for(j=0;j<np[bm];j++) if (list[bm][j]<list[bc][i]) jumpover++;
    for(j=0;j<np[wm];j++) if (list[wm][j]<list[bc][i]) jumpover++;
    decrease[bc][i]=jumpover;
  }
  /* set jumpovers white crown */
  for(i=0;i<np[wc];i++) {
    jumpover=0;
    for(j=0;j<np[bm];j++) if (list[bm][j]<list[wc][i]) jumpover++;
    for(j=0;j<np[wm];j++) if (list[wm][j]<list[wc][i]) jumpover++;
    for(j=0;j<np[bc];j++) if (list[bc][j]<list[wc][i]) jumpover++;
    decrease[wc][i]=jumpover;
  }
  /* apply decreases */
  for(i=0;i<np[bc];i++) list[bc][i]-=decrease[bc][i];
  for(i=0;i<np[wc];i++) list[wc][i]-=decrease[wc][i];
  for(i=0;i<np[wm];i++) list[wm][i]-=decrease[wm][i]+5;

  p3=combi_array[50-np[bm]-np[wm]-np[bc]][np[wc]];
  p2=p3*combi_array[50-np[bm]-np[wm]][np[bc]];
  p1=p2*combi_array[45][np[wm]];
  
  index=index_singletype(np[bm],0,45,list[bm])*p1;
  index+=index_singletype(np[wm],0,45,list[wm])*p2;
  index+=index_singletype(np[bc],0,50-np[bm]-np[wm],list[bc])*p3;
  index+=index_singletype(np[wc],0,50-np[bm]-np[wm]-np[bc],list[wc]);
  return index;
}
